#ifndef SENSORS_H
#define SENSORS_H

// TODO: Do you REALLY need the values to be doubles?

#include <GUI.h>
#include "utils.h"

typedef enum 
{
    CO,
    CO2,
    PM,
    AQ,
    NOISE,
    TEMPERATURE,
    HUMIDITY,
    VOC,

    VARIABLE_COUNT,
} Variable;

typedef enum 
{
    IAQ,
    NOISE_SENSOR,
    AQ_SENSOR,
    CO_SENSOR,
    PM_SENSOR,
    HDC1000,
    S8,

    SENSOR_COUNT,

    DEFAULT_SENSOR, // To indicate that we don't care what sensor we get the variables from
} Sensor;

// If the status is 0, the sensor is producing valid values
typedef enum 
{
    STATUS_INITIALIZING = (1 << 0),
    STATUS_PREHEATING   = (1 << 1), // TODO: remove this one?
    STATUS_STABILIZING  = (1 << 2), // TODO: remove this one?
    STATUS_ERROR        = (1 << 10),
} SensorStatus;

typedef struct
{
    float value;
    Sensor sensor;       // sensor that measured the variable
    SensorStatus status; // status of said sensor
    bool updated;        // the variable was updated at least once by this sensor since the last read
} VariableData;

void updateSensors(); // Should be called periodically (to let the sensor system perform its computations)
VariableData getVariable(Variable variable); // From the default sensor
VariableData getVariableFromSensor(Variable variable, Sensor sensor);
void readLog(float **log, uint32 *count, uint32 *currentIndex, Variable variable, Sensor sensor, uint32 seconds); // Get the last samples (`seconds` seconds worth of data)
void clearLog();

void setFanSpeed(uint8 speed);
uint8 getFanSpeed();

#endif
