/*******************************************************************************
* File Name: CO.h  
* Version 2.20
*
* Description:
*  This file contains Pin function prototypes and register defines
*
* Note:
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_PINS_CO_H) /* Pins CO_H */
#define CY_PINS_CO_H

#include "cytypes.h"
#include "cyfitter.h"
#include "cypins.h"
#include "CO_aliases.h"

/* APIs are not generated for P15[7:6] */
#if !(CY_PSOC5A &&\
	 CO__PORT == 15 && ((CO__MASK & 0xC0) != 0))


/***************************************
*        Function Prototypes             
***************************************/    

/**
* \addtogroup group_general
* @{
*/
void    CO_Write(uint8 value);
void    CO_SetDriveMode(uint8 mode);
uint8   CO_ReadDataReg(void);
uint8   CO_Read(void);
void    CO_SetInterruptMode(uint16 position, uint16 mode);
uint8   CO_ClearInterrupt(void);
/** @} general */

/***************************************
*           API Constants        
***************************************/
/**
* \addtogroup group_constants
* @{
*/
    /** \addtogroup driveMode Drive mode constants
     * \brief Constants to be passed as "mode" parameter in the CO_SetDriveMode() function.
     *  @{
     */
        #define CO_DM_ALG_HIZ         PIN_DM_ALG_HIZ
        #define CO_DM_DIG_HIZ         PIN_DM_DIG_HIZ
        #define CO_DM_RES_UP          PIN_DM_RES_UP
        #define CO_DM_RES_DWN         PIN_DM_RES_DWN
        #define CO_DM_OD_LO           PIN_DM_OD_LO
        #define CO_DM_OD_HI           PIN_DM_OD_HI
        #define CO_DM_STRONG          PIN_DM_STRONG
        #define CO_DM_RES_UPDWN       PIN_DM_RES_UPDWN
    /** @} driveMode */
/** @} group_constants */
    
/* Digital Port Constants */
#define CO_MASK               CO__MASK
#define CO_SHIFT              CO__SHIFT
#define CO_WIDTH              1u

/* Interrupt constants */
#if defined(CO__INTSTAT)
/**
* \addtogroup group_constants
* @{
*/
    /** \addtogroup intrMode Interrupt constants
     * \brief Constants to be passed as "mode" parameter in CO_SetInterruptMode() function.
     *  @{
     */
        #define CO_INTR_NONE      (uint16)(0x0000u)
        #define CO_INTR_RISING    (uint16)(0x0001u)
        #define CO_INTR_FALLING   (uint16)(0x0002u)
        #define CO_INTR_BOTH      (uint16)(0x0003u) 
    /** @} intrMode */
/** @} group_constants */

    #define CO_INTR_MASK      (0x01u) 
#endif /* (CO__INTSTAT) */


/***************************************
*             Registers        
***************************************/

/* Main Port Registers */
/* Pin State */
#define CO_PS                     (* (reg8 *) CO__PS)
/* Data Register */
#define CO_DR                     (* (reg8 *) CO__DR)
/* Port Number */
#define CO_PRT_NUM                (* (reg8 *) CO__PRT) 
/* Connect to Analog Globals */                                                  
#define CO_AG                     (* (reg8 *) CO__AG)                       
/* Analog MUX bux enable */
#define CO_AMUX                   (* (reg8 *) CO__AMUX) 
/* Bidirectional Enable */                                                        
#define CO_BIE                    (* (reg8 *) CO__BIE)
/* Bit-mask for Aliased Register Access */
#define CO_BIT_MASK               (* (reg8 *) CO__BIT_MASK)
/* Bypass Enable */
#define CO_BYP                    (* (reg8 *) CO__BYP)
/* Port wide control signals */                                                   
#define CO_CTL                    (* (reg8 *) CO__CTL)
/* Drive Modes */
#define CO_DM0                    (* (reg8 *) CO__DM0) 
#define CO_DM1                    (* (reg8 *) CO__DM1)
#define CO_DM2                    (* (reg8 *) CO__DM2) 
/* Input Buffer Disable Override */
#define CO_INP_DIS                (* (reg8 *) CO__INP_DIS)
/* LCD Common or Segment Drive */
#define CO_LCD_COM_SEG            (* (reg8 *) CO__LCD_COM_SEG)
/* Enable Segment LCD */
#define CO_LCD_EN                 (* (reg8 *) CO__LCD_EN)
/* Slew Rate Control */
#define CO_SLW                    (* (reg8 *) CO__SLW)

/* DSI Port Registers */
/* Global DSI Select Register */
#define CO_PRTDSI__CAPS_SEL       (* (reg8 *) CO__PRTDSI__CAPS_SEL) 
/* Double Sync Enable */
#define CO_PRTDSI__DBL_SYNC_IN    (* (reg8 *) CO__PRTDSI__DBL_SYNC_IN) 
/* Output Enable Select Drive Strength */
#define CO_PRTDSI__OE_SEL0        (* (reg8 *) CO__PRTDSI__OE_SEL0) 
#define CO_PRTDSI__OE_SEL1        (* (reg8 *) CO__PRTDSI__OE_SEL1) 
/* Port Pin Output Select Registers */
#define CO_PRTDSI__OUT_SEL0       (* (reg8 *) CO__PRTDSI__OUT_SEL0) 
#define CO_PRTDSI__OUT_SEL1       (* (reg8 *) CO__PRTDSI__OUT_SEL1) 
/* Sync Output Enable Registers */
#define CO_PRTDSI__SYNC_OUT       (* (reg8 *) CO__PRTDSI__SYNC_OUT) 

/* SIO registers */
#if defined(CO__SIO_CFG)
    #define CO_SIO_HYST_EN        (* (reg8 *) CO__SIO_HYST_EN)
    #define CO_SIO_REG_HIFREQ     (* (reg8 *) CO__SIO_REG_HIFREQ)
    #define CO_SIO_CFG            (* (reg8 *) CO__SIO_CFG)
    #define CO_SIO_DIFF           (* (reg8 *) CO__SIO_DIFF)
#endif /* (CO__SIO_CFG) */

/* Interrupt Registers */
#if defined(CO__INTSTAT)
    #define CO_INTSTAT            (* (reg8 *) CO__INTSTAT)
    #define CO_SNAP               (* (reg8 *) CO__SNAP)
    
	#define CO_0_INTTYPE_REG 		(* (reg8 *) CO__0__INTTYPE)
#endif /* (CO__INTSTAT) */

#endif /* CY_PSOC5A... */

#endif /*  CY_PINS_CO_H */


/* [] END OF FILE */
