/*******************************************************************************
* File Name: ILI9341_PM.c
* Version 1.70
*
* Description:
*  This file contains the setup, control and status commands to support 
*  component operations in low power mode.  
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "ILI9341.h" 

#if (CY_UDB_V0)
    static ILI9341_BACKUP_STRUCT ILI9341_backup = 
    {
        ILI9341_READ_LOW_PULSE,
        ILI9341_READ_HIGH_PULSE 
    };
#endif /* (CY_UDB_V0) */


/*******************************************************************************
* Function Name: ILI9341_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the user configuration of GraphicLCDIntf.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  ILI9341_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void ILI9341_SaveConfig(void) 
{   
    /* Saves D0/D1 Regs are non-retention for the UDB array version 0 */
    #if (CY_UDB_V0)                       
        ILI9341_backup.lowPulseWidth  = ILI9341_READ_LO_PULSE_REG;
        ILI9341_backup.highPulseWidth = ILI9341_READ_HI_PULSE_REG;      
    #endif /* CY_UDB_V0 */
}


/*******************************************************************************
* Function Name: ILI9341_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the user configuration of GraphicLCDIntf.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  ILI9341_backup - used when non-retention registers are restored.
*
*******************************************************************************/
void ILI9341_RestoreConfig(void) 
{    
    /* Restores D0/D1 Regs are non-retention for the UDB array version 0 */
    #if (CY_UDB_V0)                         
        ILI9341_READ_LO_PULSE_REG = ILI9341_backup.lowPulseWidth;
        ILI9341_READ_HI_PULSE_REG = ILI9341_backup.highPulseWidth;
    #endif /* CY_UDB_V0 */
}


/*******************************************************************************
* Function Name: ILI9341_Sleep
********************************************************************************
*
* Summary:
*  Disables block's operation and saves its configuration. Should be called 
*  prior to entering sleep.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  ILI9341_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void ILI9341_Sleep(void) 
{
    ILI9341_Stop();
    ILI9341_SaveConfig();
}


/*******************************************************************************
* Function Name: ILI9341_Wakeup
********************************************************************************
*
* Summary:
*  Enables block's operation and restores its configuration. Should be called
*  after awaking from sleep.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  ILI9341_backup - used when non-retention registers are restored.
*
*******************************************************************************/
void ILI9341_Wakeup(void) 
{
    ILI9341_RestoreConfig();
    ILI9341_Enable();
}


/* [] END OF FILE */
