/*******************************************************************************
* File Name: TP_AMux.c
* Version 1.80
*
*  Description:
*    This file contains functions for the AMuxSeq.
*
*   Note:
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "TP_AMux.h"

uint8 TP_AMux_initVar = 0u;


/*******************************************************************************
* Function Name: TP_AMux_Start
********************************************************************************
* Summary:
*  Disconnect all channels. The next time Next is called, channel 0 will be
*  connected.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void TP_AMux_Start(void)
{
    TP_AMux_DisconnectAll();
    TP_AMux_initVar = 1u;
}


/*******************************************************************************
* Function Name: TP_AMux_Init
********************************************************************************
* Summary:
*  Disconnect all channels. The next time Next is called, channel 0 will be
*  connected.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void TP_AMux_Init(void)
{
    TP_AMux_DisconnectAll();
}


/*******************************************************************************
* Function Name: TP_AMux_Stop
********************************************************************************
* Summary:
*  Disconnect all channels. The next time Next is called, channel 0 will be
*  connected.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void TP_AMux_Stop(void)
{
    TP_AMux_DisconnectAll();
}

#if (TP_AMux_MUXTYPE == TP_AMux_MUX_DIFF)

/*******************************************************************************
* Function Name: TP_AMux_Next
********************************************************************************
* Summary:
*  Disconnects the previous channel and connects the next one in the sequence.
*  When Next is called for the first time after Init, Start, Enable, Stop, or
*  DisconnectAll, it connects channel 0.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void TP_AMux_Next(void)
{
    TP_AMux_CYAMUXSIDE_A_Next();
    TP_AMux_CYAMUXSIDE_B_Next();
}


/*******************************************************************************
* Function Name: TP_AMux_DisconnectAll
********************************************************************************
* Summary:
*  This function disconnects all channels. The next time Next is called, channel
*  0 will be connected.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void TP_AMux_DisconnectAll(void)
{
    TP_AMux_CYAMUXSIDE_A_DisconnectAll();
    TP_AMux_CYAMUXSIDE_B_DisconnectAll();
}


/*******************************************************************************
* Function Name: TP_AMux_GetChannel
********************************************************************************
* Summary:
*  The currently connected channel is retuned. If no channel is connected
*  returns -1.
*
* Parameters:
*  void
*
* Return:
*  The current channel or -1.
*
*******************************************************************************/
int8 TP_AMux_GetChannel(void)
{
    return TP_AMux_CYAMUXSIDE_A_curChannel;
}

#else

/*******************************************************************************
* Function Name: TP_AMux_GetChannel
********************************************************************************
* Summary:
*  The currently connected channel is retuned. If no channel is connected
*  returns -1.
*
* Parameters:
*  void
*
* Return:
*  The current channel or -1.
*
*******************************************************************************/
int8 TP_AMux_GetChannel(void)
{
    return TP_AMux_curChannel;
}

#endif


/* [] END OF FILE */
