/********************************************************************************
* Title                 :   SenseAir_S8
* Filename              :   SenseAir_S8.c
* Author                :   JLC
* Origin Date           :   03/03/2017
* Notes                 :   None
*******************************************************************************/
/*************** MODULE REVISION LOG ******************************************
*
*    Date    Software Version    Initials       Description
*  03/03/17       1.0.0             JLC        Module Created.
*
*******************************************************************************/
/**
 * @file SenseAir_S8.c
 */
/******************************************************************************
* Includes
*******************************************************************************/

#include "SenseAir_S8.h"
#include <utils.h>

/******************************************************************************
* Module Variable Definitions
*******************************************************************************/
/*
extern uint8 cmd_read_CO2[] = {0xFE, 0X04, 0X00, 0X03, 0X00, 0X01, 0XD5, 0XC5}; //sensor address "any address", Read Input register (04), 
                                                                        //Starting adress (rgisternumber-1) 0x0003, Quantity of register (0x0001)
                                                                        //CRC calculated 0xC5D5
extern uint8 cmd_read_Status[] = {0xFE, 0X04, 0X00, 0X00, 0X00, 0X01, 0X25, 0XC5};	//sensor address "any address" (0xFE), Read Input register (04), 
                                                                        //Starting adress (rgisternumber-1) 0x0000, Quantity of register (0x0001)
                                                                        //CRC calculated 0xC5D5
extern uint8 byte_response[] = {0x0,0x0,0x0,0x0,0x0,0x0,0x0};    //Array to store the responses from the sensor
*/

/******************************************************************************
* Function Definitions
*******************************************************************************/

// It seems that "Starting adress is (rgisternumber-1)", whatever that means
// TODO: Calculate the crc automatically using startingAddress

typedef struct
{
    uint16 value;
    bool8 valid; // False if the request hasn't completed (maybe because we are still processing it or another request)
    bool8 error;
} ReadRegisterResult;

static ReadRegisterResult readRegister(uint8 startingAddress, uint16 crc)
{
    // See https://github.com/experimental-platform/example-co2/blob/master/Firmware/CO2_Sensor_ESP8266/CO2_Sensor_ESP8266.ino
    // See http://www.senseair.com/wp-content/uploads/2012/06/Modbus-on-SenseAir_R_-S8-rev_P11_1_00.pdf

    static struct
    {
        bool8 idle;
        uint8 currentAddress;

        uint8 bytesRead;
        uint8 response[7];
        uint8 timeout;
    } state = {.idle = true};

    ReadRegisterResult result = {};

    if (state.idle) // We are currently not executing any request
    {
        state.idle = false;
        state.currentAddress = startingAddress;
        state.bytesRead = 0;
        state.timeout = 100; // in milliseconds

        uint8 command[] = {0xFE, 0X04, 0X00, startingAddress, 0X00, 0X01, crc & 0xff, (crc >> 8) & 0xff};
        // 0xFE: Sensor address ("any address")
        // 0x04: Read Input Registers
        // 0x0100: "Quantity of register" (edu: ???? This must be the number of registers we want to read, I guess)

        UART_S8_ClearRxBuffer();
        UART_S8_ClearTxBuffer();
        UART_S8_PutArray(command, arrayCount(command));
    }
    else if (state.currentAddress != startingAddress) // We are working on somebody else's request
    {
        return result;
    }

    // Reading byte by byte
    while (state.bytesRead < arrayCount(state.response))
    {
        if (UART_S8_GetRxBufferSize() == 0)
        {
            state.timeout--;
            if (state.timeout == 0)
            {
                state.idle = true;
                result.error = true;
            }

            // OK, well done. We'll continue later on, if we haven't timed out.
            return result;
        }

        state.response[state.bytesRead++] = UART_S8_ReadRxData(); // TODO: error handling
    }

    result.error = true;
    if (state.response[0] != 0xfe) return result; // 0xfe: any address
    if (state.response[1] != 0x04) return result; // 0x04: read input register
    if (state.response[2] != 0x02) return result; // 0x02: Número de bytes de datos enviados
    result.value = state.response[3]*256 + state.response[4]; //Combine high byte and low byte with this formula to get value

    // response[5] and response[6] hold the CRC. We aren't using it right now

    result.error = false;
    result.valid = true;

    state.idle = true;

    return result;
}

/*******************************************************************************
* Function Name  : get_co2_S8
* Description    : Obtiene el CO2 medido por el sensor S8
* Input          : None
*
*
*******************************************************************************/
uint16 get_co2_S8(void)
{	
    uint16 co2Val = S8_READ_INCOMPLETE;

    ReadRegisterResult registerResult = readRegister(0x03, 0xC5D5);

    if (registerResult.valid)
    {
        co2Val = registerResult.value;
    }

    if (registerResult.error)
    {
        co2Val = S8_READ_ERROR;
    }

    return co2Val;
}

/*******************************************************************************
* Function Name  : get_status_S8
* Description    : Obtiene el status del sensor S8
* Input          : None
*
*
* return=      0x00, sensor OK
               0x01, fatal error
               0x02, Offset regulator Error
               0x04, Algorithm Error
               0x08, Output Error
               0x10, Self Diagnosis
               0x20, Out of range
               0x40, Memory Error
*******************************************************************************/
uint16 get_status_S8(void)
{	
    // TODO: Not really tested!

    uint16 status = S8_READ_INCOMPLETE;

    ReadRegisterResult registerResult = readRegister(0x00, 0xC525);

    if (registerResult.valid)
    {
        status = registerResult.value;
    }

    if (registerResult.error)
    {
        status = S8_READ_ERROR;
    }

    return status;
}// End of Function

/* [] END OF FILE */
    
    
