// TODO ?: this file was just a quick hack

#include <device.h>
#include <stdio.h>
#include "cypins.h"

#include <utils.h>
#include "tft.h"
#include "debug.h"

void initDebug()
{
    UART_Start();
}

char putByte(char c)
{
    UART_PutChar((uint8)c);
    return c;
}

void putString(char *str)
{
    UART_PutString(str);
}

void uartPrint(const char *format, ...)
{
    static char buffer[250];

    va_list args;
    va_start(args, format);
    vsnprintf(buffer, sizeof(buffer), format, args);
    va_end(args);

    UART_PutString(buffer);
}

// TODO: volatile
volatile struct
{
    char data[128];
    uint8_t count;

    int readIndex;
    int writeIndex;

    int errorStatus;
    int overrun;
} rxBuffer = {};


CY_ISR(RxIsr)
{
    uint8 rxStatus;         
    uint8 rxData;           

    do
    {
        if (rxBuffer.count == arrayCount(rxBuffer.data))
        {
            rxBuffer.overrun = true;
            break;
        }

        /* Read receiver status register */
        rxStatus = UART_RXSTATUS_REG;

        if((rxStatus & (UART_RX_STS_BREAK      | UART_RX_STS_PAR_ERROR |
                        UART_RX_STS_STOP_ERROR | UART_RX_STS_OVERRUN)) != 0u)
        {
            /* ERROR handling. */
            rxBuffer.errorStatus |= rxStatus & ( UART_RX_STS_BREAK      | UART_RX_STS_PAR_ERROR | 
                                                 UART_RX_STS_STOP_ERROR | UART_RX_STS_OVERRUN);
        }
        
        if((rxStatus & UART_RX_STS_FIFO_NOTEMPTY) != 0u)
        {
            /* Read data from the RX data register */
            rxData = UART_RXDATA_REG;
            if(rxBuffer.errorStatus == 0u)
            {
                rxBuffer.data[rxBuffer.writeIndex] = rxData;
                rxBuffer.writeIndex = (rxBuffer.writeIndex + 1) % arrayCount(rxBuffer.data);
                rxBuffer.count++;
            }
        }

    } while((rxStatus & UART_RX_STS_FIFO_NOTEMPTY) != 0u);

}

int inited = false;
char getByte()
{
    // TODO: I don't like any of this

    if (!inited)
    {
        Debug_Rx_ISR_StartEx(RxIsr);
        CyGlobalIntEnable;
        inited = true;
    }

    while (rxBuffer.count == 0) // TODO: is this safe?
        ;

    Debug_Rx_ISR_Disable();
      int status = rxBuffer.errorStatus;
      rxBuffer.errorStatus = 0;
      rxBuffer.count--;
    Debug_Rx_ISR_Enable();

    uint8_t byte = rxBuffer.data[rxBuffer.readIndex];
    rxBuffer.readIndex = (rxBuffer.readIndex + 1) % arrayCount(rxBuffer.data);

#if 0
    char buffer_[500];
    sprintf(buffer_, "getByte(): status: 0x%x byte: 0x%x (%c)\r\n", status, byte, byte);
    putString(buffer_);
#endif

    if (status != 0)
    {
        if (status & UART_RX_STS_PAR_ERROR)      putString("getByte(): par error\r\n");
        if (status & UART_RX_STS_STOP_ERROR)     putString("stop error\r\n");
        if (status & UART_RX_STS_BREAK)          putString("break error\r\n");
        if (status & UART_RX_STS_OVERRUN)        putString("overrun error\r\n");
        if (status & UART_RX_STS_ADDR_MATCH)     putString("addr match error\r\n");
        if (status & UART_RX_STS_MRKSPC)         putString("MRKSPC error\r\n");
        if (status & UART_RX_STS_SOFT_BUFF_OVER) putString("soft buffer overrun error\r\n");

        //char *statusMsg = "undocumented status\r\n";
        //sprintf(buffer_, "getByte(): 0x%x (%s)\r\n", status, statusMsg);
        //putString(buffer_);
    }

    return byte;
}

#define ZERO_PAD_COUNT 2
char buffer[100];
char *errorMessage = 0;
char errorMessageBuffer[100]; // in case you want to sprintf

// TODO: getLine -> getCommandString
int getLine(char **line, int *length)
{
    int success = true;
    int count = 0;
    *line = buffer;

    for ( ; ; )
    {
        int c = getByte();

        if ((c >= 0) && (c <= 0x7f))
        {
            if ((c == ';') || (c == '\n') || (c == '\r'))
            {
                if (count > 0)
                {
                    break;
                }
            }
            else
            {
                if (count + ZERO_PAD_COUNT < arrayCount(buffer))
                {
                    buffer[count++] = c;
                }
                else
                {
                    errorMessage = "oops! line too long\r\n";
                    success = false;
                    break;
                }
            }
        }
        else
        {
            errorMessage = "weird character in line\r\n";
            success = false;
            break;
        }
    }

    for (int i = 0; i < ZERO_PAD_COUNT; i++)
    {
        buffer[count + i] = 0;
    }


    *line = buffer;
    *length = count;

#if 0
    char buffer_[500];
    sprintf(buffer_, "%s %d %s\r\n", buffer, count, success ? "success" : "failure");
    putString(buffer_);
#endif

    return success;
}

int isSeparator(char c)
{
    return (c == ' ');
}

int isWordEnd(char c)
{
    return isSeparator(c) || (c == 0);
}

int isHexDigit(char c)
{
    int isIs = ((c >= '0') && (c <= '9')) ||
               ((c >= 'a') && (c <= 'f')) ||
               ((c >= 'A') && (c <= 'F'));
    return isIs;
}

int getBit(char c)
{
    if ((c == '0') || (c == '1')) return c - '0';
    else return -1;
}

int getDigit(char c)
{
    if ((c >= '0') && (c <= '9')) return c - '0';
    else return -1;
}

int getHexDigit(char c)
{
    if      ((c >= '0') && (c <= '9')) return c - '0';
    else if ((c >= 'a') && (c <= 'f')) return (c - 'a') + 10;
    else if ((c >= 'A') && (c <= 'F')) return (c - 'A') + 10;
    else return -1;
}

int isWordCharacter(char c)
{
    return (c && (c != ' '));
}

char *eatSeparators(char *line)
{
    while (isSeparator(*line))
        line++;
    return line;
}

char *eatWord(char *line)
{
    while (isWordCharacter(*line))
        line++;
    return line;
}

typedef enum { READ, WRITE, DUMP_MEMORY } CommandType;

typedef struct
{
    CommandType type;
    uint8_t reg;
    int byteCount;
    int parameters[20];
} Command;

void printByte(uint8_t val)
{
    for (int i = 0; i < 8; i++)
    {
        putByte(((val >> (7 - i)) & 1) ? '1' : '0');
        if (i == 3)
            putByte(' ');
    }
    putByte('\n');
}


uint8_t bit(uint8_t byte, uint8_t bitNumber)
{
    uint8_t result = (byte >> bitNumber) & 1;
    return result;
}

uint8_t bits(uint8_t byte, uint8_t higher, uint8_t lower)
{
    if (higher < lower)
    {
        uint8_t tmp = higher; higher = lower; lower = tmp;
    }

    uint8_t bitsToSetInMask = (higher - lower) + 1;
    uint8_t mask = (1 << bitsToSetInMask) - 1;
    uint8_t result = (byte >> lower) & mask;

    return result;
}

void dumpDisplayMemory(uint16_t x0, uint16_t y0, uint16_t w, uint16_t h)
{
    char buffer_[500];

    uint16_t x1 = x0 + w - 1;
    uint16_t y1 = y0 + h - 1;

    sprintf(buffer_, "[Display memory (%d, %d) -> (%d, %d)]\nP3\n%d %d\n255\n", x0, y0, x1, y1, w, h);
    putString(buffer_);

    setAddrWindow(x0, y0, x1, y1);

    tftWriteCommand(0x2e);
    tftReadData(); // dummy, skip

    for (int y = 0; y < h; y++)
    {
        for (int x = 0; x < w; x++)
        {
            uint8_t r = tftReadData();
            uint8_t g = tftReadData();
            uint8_t b = tftReadData();

            sprintf(buffer_, "%.3d %.3d %.3d  ", r, g, b);
            putString(buffer_);
        }
        putString("\n");
    }
}

// Read Display Status (RDDST) (p. 92)
void printRDDST(int number, uint8_t byte)
{
    char *indentation = "        ";
    //char *separator = "  --  ";
    char *separator = "  |  ";
    switch (number)
    {
        case 1:
        {
            putString(indentation);

            // bit 31 (7)
            if (!bit(byte, 7))  putString("Booster OFF");
            else                putString("Booster ON");
            putString(separator);

            // bit 30 (6)
            if (!bit(byte, 6)) putString("Top to bottom");
            else               putString("Bottom to top");
            putString(separator);

            // bit 29 (5)
            if (!bit(byte, 5)) putString("Left to right");
            else               putString("Right to left");
            putString(separator);

            // bit 28 (4)
            if (!bit(byte, 4)) putString("Normal mode");
            else               putString("Reverse mode");

            putString("\n");

            putString(indentation);

            // bit 27 (3)
            if (!bit(byte, 3)) putString("Refresh top to bottom");
            else               putString("Refresh bottom to top");
            putString(separator);

            // bit 26 (2)
            if (!bit(byte, 2)) putString("RGB");
            else               putString("BGR");
            putString(separator);

            // bit 25 (1)
            if (!bit(byte, 1)) putString("Refresh left to right");
            else               putString("Refresh right to left");
            putString(separator);

            // bit 24 (0)
            if (!bit(byte, 0)) putString("OK");
            else               putString("E"); // should be 0

            putString("\n");
        } break;

        case 2:
        {
            putString(indentation);

            // bit 23 (7)
            if (!bit(byte, 7)) putString("OK");
            else               putString("E"); // should be 0
            putString(separator);

            // bit 22, 21, 20 (6, 5, 4)
            if      (bits(byte, 6, 4) == 5) putString("16 bit/pixel");
            else if (bits(byte, 6, 4) == 4) putString("18 bit/pixel");
            else                            putString("E");
            putString(separator);

            // bit 19 (3)
            if (!bit(byte, 3)) putString("Idle mode OFF");
            else               putString("Idle mode ON");

            putString("\n");

            putString(indentation);

            // bit 18 (2)
            if (!bit(byte, 2)) putString("Partial mode OFF");
            else               putString("Partial mode ON");
            putString(separator);

            // bit 17 (1)
            if (!bit(byte, 1)) putString("Sleep IN mode");
            else               putString("Sleep OUT mode");
            putString(separator);

            // bit 16 (0)
            if (!bit(byte, 0)) putString("Display normal mode OFF");
            else               putString("Display normal mode ON");

            putString("\n");
        } break;

        case 3:
        {
            putString(indentation);

            if (bits(byte, 7, 3) == 0) putString("OK");
            else                       putString("E");
            putString(separator);

            // bit 10 (2)
            if (!bit(byte, 2)) putString("Display OFF");
            else              putString("Display ON");
            putString(separator);

            // bit 9 (1)
            if (!bit(byte, 2)) putString("Tearing effect line OFF");
            else              putString("Tearing effect line ON");

            // bit 8 - 6 (ooops)

            putString("\n");
        } break;

        case 4:
        {
            putString(indentation);

            // bit 5 (5)
            if (!bit(byte, 5)) putString("Tearing effect Mode 1");
            else               putString("Tearing effect Mode 2");
            putString(separator);

            for (int i = 4; i >= 0; i--)
            {
                if (!bit(byte, i)) putString("OK");
                else               putString("E"); // should be 0

                if (i > 0) putString(separator);
            }

            putString("\n");
        } break;
    }
}


void executeCommand(Command command)
{
    if (command.type == READ)
    {

    char buffer_[500];
    sprintf(buffer_, "reading %d bytes from register 0x%02x\r\n", command.byteCount, command.reg);
    putString(buffer_);

        void (*prettyPrint)(int number, uint8_t byte) = 0;
        if (command.reg == 0x09) prettyPrint = printRDDST;

        tftWriteCommand(command.reg);
        for (int i = 0; i < command.byteCount; i++)
        {
            uint8_t byte = tftReadData();
            printByte(byte);
            if (prettyPrint) prettyPrint(i, byte);
        }
    }
    else if (command.type == WRITE)
    {
    char buffer_[500];
    sprintf(buffer_, "writing to register 0x%02x\r\n", command.reg);
    putString(buffer_);

        tftWriteCommand(command.reg);
        for (int i = 0; i < command.byteCount; i++)
        {
            tftWriteData(command.parameters[i]);
            printByte(command.parameters[i]);
        }
    }
    else if (command.type == DUMP_MEMORY)
    {
        dumpDisplayMemory(command.parameters[0], command.parameters[1], command.parameters[2], command.parameters[3]);
    }
}

int defaultNumberOfParameters(uint8_t reg)
{
    if (reg == 0x09)
    {
        return 5;
    }

    return 0;
}

bool8 exited = false;

int parseAndExecuteNextLine()
{
    int count;
    char *line;
    if (getLine(&line, &count) && (count > 0))
    {
        putString("> ");
        putString(line);
        putString("\r\n");

        line = eatSeparators(line);
        Command command;

        // R/W //
        if (*line == 'r')
        {
            command.type = READ;
        }
        else if (*line == 'w')
        {
            command.type = WRITE;
        }
        else if (*line == 'd')
        {
            command.type = DUMP_MEMORY;
        }
        else if (*line == 'c')
        {
            putString("continuing\n");
            return true;
        }
        else if (*line == 'e')
        {
            putString("exiting shell\n");
            exited = true;
            return true;
        }
        else
        {
            putString("error: expected 'r', 'w', 'd.*' or 'e.*'\r\n");
            return false;
        }

        line = eatWord(line);
        line = eatSeparators(line);

        // Address //
        if (command.type == WRITE || command.type == READ)
        {
            command.reg = 0x00;

            if (!*line)
            {
                putString("error: missing address\r\n");
                return false;
            }

            if ((line[0] == '0') && (line[1] == 'x')) // the 0x in 0xDD
                line += 2;

            int digitVal;
            while ((digitVal = getHexDigit(*line)) >= 0)
            {
                command.reg = digitVal + command.reg * 16;
                line++;
            }

            if (!isWordEnd(*line))
            {
                putString("error: error in address\r\n");
                return false;
            }

            line = eatSeparators(line);
        }


        // Number of bytes for READ //
        if (command.type == READ)
        {
            command.byteCount = defaultNumberOfParameters(command.reg);

            if (!*line && command.byteCount < 0)
            {
                putString("error: missing number of bytes\r\n");
            }

            if (*line)
            {
                int digitVal = 0;
                while ((digitVal = getDigit(*line)) >= 0)
                {
                    command.byteCount = digitVal + command.byteCount * 10;
                    line++;
                }

                if (!isWordEnd(*line))
                {
                    putString("error: error in number of bytes\r\n");
                    return false;
                }

                line = eatSeparators(line);

            }
        }

        // Parameters for WRITE // (maybe "arguments"?)
        if (command.type == WRITE)
        {
#if 0
            if (command.byteCount > arrayCount(command.parameters))
            {
                putString("error: I'm just supporting a few parameters per command\r\n");
                putString("  (not gonna tell you how many until I can printf)\r\n");
                return false;
            }
#endif

            command.byteCount = 0;
            int error = false;
            for ( ; !error; command.byteCount++)
            {
                if (!*line)
                {
                    break;
                }

                if (command.byteCount + 1 > arrayCount(command.parameters))
                {
                    putString("error: I'm just supporting a few parameters per command\r\n");
                    putString("  (not gonna tell you how many until I can printf)\r\n");
                    return false;
                }

                if ((line[0] == '0') && (line[1] == 'x'))
                {
                    // Hexadecimal
                    line += 2;
                    int digitVal;
                    while ((digitVal = getHexDigit(*line)) >= 0)
                    {
                        command.parameters[command.byteCount] = digitVal + command.parameters[command.byteCount] * 16;
                        line++;
                    }
                }
                else
                {
                    // Binary

                    int digitVal = 0;
                    command.parameters[command.byteCount] = 0;
                    while (*line == '_') line++;
                    while ((digitVal = getDigit(*line)) >= 0)
                    {
                        command.parameters[command.byteCount] = digitVal + command.parameters[command.byteCount] * 2;
                        line++;
                        while (*line == '_') line++;
                    }
                }

                if (!isWordEnd(*line))
                {
                    putString("error: error in parameter\r\n");
                    return false;
                }

                line = eatSeparators(line);
            }

            if (error)
            {
                putString("error: error in parameter\r\n");
                return false;
            }
        }

        // Arguments for DUMP_MEMORY //
        if (command.type == DUMP_MEMORY)
        {
            command.parameters[0] = 0;
            command.parameters[1] = 0;
            command.parameters[2] = 100;
            command.parameters[3] = 100;

            int error = false;
            int argCount = 0;
            for ( ; !error; argCount++)
            {
                if (!*line)
                {
                    break;
                }

                if (argCount + 1 > 4)
                {
                    putString("error: too many arguments\n");
                    return false;
                }

                // Decimal
                int digitVal;
                command.parameters[argCount] = 0;
                while ((digitVal = getDigit(*line)) >= 0)
                {
                    command.parameters[argCount] = digitVal + command.parameters[argCount] * 10;
                    line++;
                }

                if (!isWordEnd(*line))
                {
                    putString("error: error in argument\r\n");
                    return false;
                }

                line = eatSeparators(line);
            }

            if (error)
            {
                putString("error: error in argument\r\n");
                return false;
            }
        }

        if (*line)
        {
            putString("error: extra characters\r\n");
            return false;
        }

        executeCommand(command);
    }

    return false; // not done
}

void myshell()
{
    if (exited) return;

    putString("> \r\n");
    int done = false;
    while (!done)
    {
        done = parseAndExecuteNextLine();
    }
}
