/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2009  SEGGER Microcontroller GmbH & Co. KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

** emWin V5.00 - Graphical user interface for embedded applications **
emWin is protected by international copyright laws.   Knowledge of the
source code may not be used to write a similar product.  This file may
only be used in accordance with a license and should not be re-
distributed in any way. We appreciate your understanding and fairness.
----------------------------------------------------------------------
File        : LCDConf_CompactColor_16.h
Purpose     : CompactColor driver configuration file
----------------------------------------------------------------------
*/

#ifndef LCDCONF_COMPACTCOLOR_16_H
#define LCDCONF_COMPACTCOLOR_16_H

#include <project.h>

/*********************************************************************
*       General configuration of LCD
*********************************************************************/
 
/* LCD Controller to use by emWin */
#define LCD_CONTROLLER      66709

/* LCD orientation settings */ 
#define LCD_MIRROR_X            0
#define LCD_MIRROR_Y            0
#define LCD_SWAP_XY             1

/* Color depths - bits per pixel and bit order */
#define LCD_BITSPERPIXEL        16
#define LCD_SWAP_RB             1

/* 16 bit parallel interface is used */
#define LCD_USE_PARALLEL_16     0


/*********************************************************************
*
*                   Simple bus configuration
*
**********************************************************************
*/
#if (LCD_USE_PARALLEL_16 == 1)
    void LCD_Intf16_WriteM01(unsigned short * pData, int NumWords);
    void LCD_Intf16_WriteM00(unsigned short * pData, int NumWords);
    void LCD_Intf16_ReadM01(unsigned short * pData, int NumWords);
        
    #define LCD_WRITE_A0(Word)         CYGRAPHICS_WRITE(0, Word)    
    #define LCD_WRITE_A1(Word)         CYGRAPHICS_WRITE(1, Word)
    #define LCD_WRITEM_A0(p, NumWords) LCD_Intf16_WriteM00(p, NumWords)
    #define LCD_WRITEM_A1(p, NumWords) LCD_Intf16_WriteM01(p, NumWords)    
    #define LCD_READ_A0                CYGRAPHICS_READ(0)
    #define LCD_READ_A1                CYGRAPHICS_READ(1)
    #define LCD_READM_A1(p, NumWords)  LCD_Intf16_ReadM01(p, NumWords)
#else /* 8-bit parallel interface */
    void LCD_Intf8_WriteM01(unsigned char * pData, int NumBytes);
    void LCD_Intf8_WriteM00(unsigned char * pData, int NumBytes);
    void LCD_Intf8_ReadM01(unsigned char * pData, int NumBytes);
        
    #define LCD_WRITE_A0(Byte)         CYGRAPHICS_WRITE(0, Byte)    
    #define LCD_WRITE_A1(Byte)         CYGRAPHICS_WRITE(1, Byte)
    #define LCD_WRITEM_A1(p, NumBytes) LCD_Intf8_WriteM01(p, NumBytes)
    #define LCD_WRITEM_A0(p, NumBytes) LCD_Intf8_WriteM00(p, NumBytes)    
    #define LCD_READ_A0                CYGRAPHICS_READ(0)
    #define LCD_READ_A1                CYGRAPHICS_READ(1)
    #define LCD_READM_A1(p, NumBytes)  LCD_Intf8_ReadM01(p, NumBytes)    
#endif       

#endif /* LCDCONF_COMPACTCOLOR_16_H */

/*************************** End of file ****************************/
