#include <I2C.h>
#include "i2c_communication.h"

static bool8 error = false;
static uint8 internalBuffer[8];
static Timestamp timestamp;

void i2cStart()
{
    error = false;
    I2C_Start();
    timestamp = getTimestamp();
}

void i2cBeginTransaction()
{
    I2C_MasterClearStatus();
    error = false;
}

bool8 i2cEndTransaction()
{
    return error;
}

uint8 i2cGetMasterStatus()
{
    return I2C_MasterStatus();
}

void i2cWrite(uint8 address, uint8 *buffer, int count)
{
    uartPrint("i2cWrite: address = 0x%0x, error = %s\n", address, error ? "true" : "false");

    if (error) return;

    I2C_MasterClearStatus();

    uint8 ret = I2C_MasterWriteBuf(address, buffer, count, I2C_MODE_COMPLETE_XFER);
    if (ret != I2C_MSTR_NO_ERROR)
    {
        error = true;
        return;
    }

    timestamp = getTimestamp();
    while (!(I2C_MasterStatus() & I2C_MSTAT_WR_CMPLT))
    {
        if (msSince(timestamp) > TIMEOUT_IN_MS)
        {
            error = true;
            return;
        }
    }

    if (I2C_MasterStatus() & I2C_MSTAT_ERR_XFER)
    {
        error = true;
    }
}

void i2cWriteByte(uint8 address, uint8 byte)
{
    internalBuffer[0] = byte;
    i2cWrite(address, internalBuffer, 1);
}

void i2cWriteU16(uint8 address, uint16 value)
{
    internalBuffer[0] = value >> 8;
    internalBuffer[1] = value & 8;
    i2cWrite(address, internalBuffer, 2);
}

void i2cRead(uint8 address, uint8 *buffer, int count)
{
    uartPrint("i2cRead: address = 0x%0x, error = %s\n", address, error ? "true" : "false");

    if (error) return;

    I2C_MasterClearStatus();

    uint8 ret = I2C_MasterReadBuf(address, buffer, count, I2C_MODE_COMPLETE_XFER);
    if (ret != I2C_MSTR_NO_ERROR)
    {
        error = true;
        return;
    }

    timestamp = getTimestamp();
    while (!(I2C_MasterStatus() & I2C_MSTAT_RD_CMPLT))
    {
        if (msSince(timestamp) > TIMEOUT_IN_MS)
        {
            error = true;
            return;
        }
    }

    if (I2C_MasterStatus() & I2C_MSTAT_ERR_XFER)
    {
        error = true;
    }
}
