// NOTE: I've added a layer of indirection in case I want to replace the Mikroelektronika code with my own.

#include "cypins.h"
#include "tft.h"

uint8 backlightLevel; // from 0 to 100

void setBacklight(uint8 level)
{
    backlightLevel = level;
    level = 255 * (float)backlightLevel / 100;

    PWM_BLED_WriteCompare(backlightLevel);
}

uint8 getBacklight()
{
    return backlightLevel;
}

void tftWriteCommand(uint8 value)
{
    ILI9341_Write8(0, value);
}

void tftWriteData(uint8 value)
{
    ILI9341_Write8(1, value);
}

uint8 tftReadData()
{
    return ILI9341_Read8(1);
}

static void tftSetOrientation(uint8 orientation)
{
	tftWriteCommand(0x36);
	switch (orientation)
	{
		case 0: tftWriteData(0x48);
				break;
		case 1: tftWriteData(0x28);
				break;
		case 2: tftWriteData(0x88);
				break;
		case 3: tftWriteData(0xE8);
				break;
	}
}

// TODO: Incomplete
void tftInit(uint8 orientation)
{
    PWM_BLED_Start(); // Should clear the compare register (and turn off the display)
    backlightLevel = 100 * (float)(PWM_BLED_ReadCompare() / 255);

    ILI9341_Start();

    tftWriteCommand(0x01);
    CyDelay(100);

#if 1
    //************* Start Initial Sequence **********//
    //Power control A
    tftWriteCommand  (0xCB);
    tftWriteData(0x39);
    tftWriteData(0x2C);
    tftWriteData(0x00);
    tftWriteData(0x34);
    tftWriteData(0x02);

    //Power control A
    tftWriteCommand  (0xCF);
    tftWriteData(0x00);
    tftWriteData(0XC1);
    tftWriteData(0X30);

    //Driver timing control A
    tftWriteCommand  (0xE8);
    tftWriteData(0x85);
    tftWriteData(0x00);
    tftWriteData(0x78);

    //Driver timing control B
    tftWriteCommand  (0xEA);
    tftWriteData(0x00);
    tftWriteData(0x00);

    //Power on sequence control
    tftWriteCommand(0xED);
    tftWriteData(0x64);
    tftWriteData(0x03);
    tftWriteData(0X12);
    tftWriteData(0X81);

    //Pump ratio control
    tftWriteCommand (0xF7);
    tftWriteData(0x20);

    //Power Control 1
    tftWriteCommand  (0xC0);
    tftWriteData(0x23);

    //Power Control 2
    tftWriteCommand  (0xC1);
    tftWriteData(0x10);

    //VCOM Control 1
    tftWriteCommand(0xC5);
    tftWriteData(0x2B);
    tftWriteData(0x2B);
#endif

    //Memory Access Control
    tftSetOrientation(orientation);

    //Frame Rate Control (In Normal Mode/Full Colors)
    tftWriteCommand(0xB1);
    tftWriteData(0x00);
    tftWriteData(0x1B);	// 10

#if 1
    //Display Function Control
    tftWriteCommand(0xB6);
    tftWriteData(0x0A);
    tftWriteData(0x02);	// A2

#if 0
    //Enable 3G
    tftWriteCommand(0xF2);
    tftWriteData(0x02);  //off
#endif

    //COLMOD: Pixel Format Set
    tftWriteCommand(0x3a);
    tftWriteData(0x05);

    //Gamma Set
    tftWriteCommand(0x26);   //Gamma curve 3
    tftWriteData(0x01);

    //Positive Gamma Correction
    tftWriteCommand(0xE0);
    tftWriteData(0x0F);
    tftWriteData(0x31);
    tftWriteData(0x2B);
    tftWriteData(0x0C);
    tftWriteData(0x0E);
    tftWriteData(0x08);
    tftWriteData(0x4E);
    tftWriteData(0xF1);
    tftWriteData(0x37);
    tftWriteData(0x07);
    tftWriteData(0x10);
    tftWriteData(0x03);
    tftWriteData(0x0E);
    tftWriteData(0x09);
    tftWriteData(0x00);

    //Negative Gamma Correction
    tftWriteCommand(0XE1);
    tftWriteData(0x00);
    tftWriteData(0x0E);
    tftWriteData(0x14);
    tftWriteData(0x03);
    tftWriteData(0x11);
    tftWriteData(0x07);
    tftWriteData(0x31);
    tftWriteData(0xC1);
    tftWriteData(0x48);
    tftWriteData(0x08);
    tftWriteData(0x0F);
    tftWriteData(0x0C);
    tftWriteData(0x31);
    tftWriteData(0x36);
    tftWriteData(0x0F);

    // Display inversion ON
    tftWriteCommand(0x21); // TODO: ??? Why do we suddenly need this?
#endif

    // Sleep Out
    tftWriteCommand(0x11);
    CyDelay(120);
    //Display On
    tftWriteCommand(0x29);

    // Set refresh rate
    tftWriteCommand(0xb1);
    tftWriteData(0x01); // the last two bits determine the divider
    tftWriteData(0x1b);
}

void setAddrWindow(uint16 x0, uint16 y0, uint16 x1, uint16 y1)
{
  tftWriteCommand(ILI9341_CASET); // Column addr set
  tftWriteData(x0 >> 8);
  tftWriteData(x0 & 0xFF);     // XSTART 
  tftWriteData(x1 >> 8);
  tftWriteData(x1 & 0xFF);     // XEND

  tftWriteCommand(ILI9341_PASET); // Row addr set
  tftWriteData(y0>>8);
  tftWriteData(y0);     // YSTART
  tftWriteData(y1>>8);
  tftWriteData(y1);     // YEND

  tftWriteCommand(ILI9341_RAMWR); // write to RAM
}

#if 0
#include <stdio.h>
void tftDot(int16 x, int16 y, uint16 color)
{
    // TODO: Clipping?

    setAddrWindow(x, y, x, y);
    tftWriteData(color >> 8);
    tftWriteData(color & 0xff);
}

/*
void tftFillRect(int16 x, int16 y, int16 w, int16 h, uint32 color)
{
    uint8 r = (color >> 16) & 0xff;
    uint8 g = (color >>  8) & 0xff;
    uint8 b = (color >>  0) & 0xff;
    // TODO: this is too confusing, and probably buggy
    uint8 msb = (r & 0xf8) | (g >> 5);
    uint8 lsb = ((g << 3) & 0xe0) | (b >> 3);

    // rudimentary clipping (drawChar w/big text requires this)
    if((x >= TFT_WIDTH) || (y >= TFT_HEIGHT)) return;
    if((x + w - 1) >= TFT_WIDTH)  w = TFT_WIDTH  - x;
    if((y + h - 1) >= TFT_HEIGHT) h = TFT_HEIGHT - y;

    setAddrWindow(x, y, x+w-1, y+h-1);

    for(y=h; y>0; y--) {
        for(x=w; x>0; x--) {
            tftWriteData(msb);
            tftWriteData(lsb);
        }
    }

    tftWriteCommand(ILI9341_NOP); // just in case?
}
*/

void tftFillRect(int16 x0, int16 y0, int16 x1, int16 y1, uint32 color)
{
    uint8 r = (color >> 16) & 0xff;
    uint8 g = (color >>  8) & 0xff;
    uint8 b = (color >>  0) & 0xff;
    // TODO: this is too confusing, and probably buggy
    uint8 msb = (r & 0xf8) | (g >> 5);
    uint8 lsb = ((g << 3) & 0xe0) | (b >> 3);

    // clipping
    if (x0 < 0) x0 = 0;
    if (y0 < 0) y0 = 0;
    if (x1 > TFT_WIDTH)  x1 = TFT_WIDTH;
    if (y1 > TFT_HEIGHT) y1 = TFT_HEIGHT;

    if ((x0 >= x1) || (y0 >= y1)) return;

    setAddrWindow(x0, y0, x1 - 1, y1 - 1);

    for (int16 j = y0; j < y1; j++)
    {
        for (int16 i = x0; i < x1; i++)
        {
            tftWriteData(msb);
            tftWriteData(lsb);
        }
    }

    tftWriteCommand(ILI9341_NOP); // just in case?
}

static char buffer[32];

int tftPrint(uint16 x, uint16 y, const char *format, ...)
{
    va_list args;
    va_start(args, format);
    int result = vsnprintf(buffer, sizeof(buffer), format, args);
    va_end(args);

    TFT_Write_Text((uint8 *)buffer, x, y);
    //TFT_DrawString(buffer, col, row, 2, ILI9341_WHITE);

    return result;
}

int tftVPrint(uint16 x, uint16 y, const char *format, va_list args)
{
    int result = vsnprintf(buffer, sizeof(buffer), format, args);

    TFT_Write_Text((uint8 *)buffer, x, y);
    //TFT_DrawString(buffer, col, row, 2, ILI9341_WHITE);

    return result;
}

// Temporal
int tftPrintCentered(int16 x0, int16 y0, int16 x1, int16 y1, int16 fontSizeX, int16 fontSizeY, const char *format, ...)
{
    va_list args;
    va_start(args, format);
    int length = vsnprintf(buffer, sizeof(buffer), format, args);
    va_end(args);

    if (length > 0)
    {
        //int16 x = x0 + ((x1 - x0) - fontSizeX * length) / 2;
        int16 x = x0;
        int16 y = y0 + ((y1 - y0) - fontSizeY) / 2;
        TFT_Write_Text((uint8 *)buffer, x, y);
    }

    return length;
}

void drawBitmap(int16 topLeftX, int16 topLeftY, Bitmap bmp)
{
    // TODO: Clipping

    setAddrWindow(topLeftX, topLeftY, topLeftX + bmp.width - 1, topLeftY + bmp.height - 1);

    uint8 *pixel = bmp.image;
    for (int16 i = 0; i < bmp.height; i++)
    {
        for (int16 j = 0; j < bmp.width; j++)
        {
            uint16 color = ILI9341_RED; // should never be seen

            if (*pixel < bmp.numberOfColors)
            {
                color = bmp.palette[*pixel];
            }

            tftWriteData(color >> 8);
            tftWriteData(color & 0xff);

            pixel++;
        }
    }

    tftWriteCommand(ILI9341_NOP); // just in case?

    // Something weird happens when I call Mikroelektronika TFT_* functions if I don't do this.
    setAddrWindow(0, 0, TFT_WIDTH - 1, TFT_HEIGHT - 1);
    tftWriteCommand(ILI9341_NOP);
}

void drawRawBitmap(int16 topLeftX, int16 topLeftY, RawBitmap bmp)
{
    // TODO: Clipping

    setAddrWindow(topLeftX, topLeftY, topLeftX + bmp.width - 1, topLeftY + bmp.height - 1);

    uint16 *row = bmp.image;
    for (int16 i = 0; i < bmp.height; i++)
    {
        uint16 *pixel = row;
        for (int16 j = 0; j < bmp.width; j++)
        {
            uint16 color = *pixel;

            tftWriteData(color >> 8);
            tftWriteData(color & 0xff);

            pixel++;
        }
        row += bmp.stride;
    }

    tftWriteCommand(ILI9341_NOP); // just in case?

    // Something weird happens when I call Mikroelektronika TFT_* functions if I don't do this.
    setAddrWindow(0, 0, TFT_WIDTH - 1, TFT_HEIGHT - 1);
    tftWriteCommand(ILI9341_NOP);
}

GUI_DRAW_FUNCTION(guiDrawFunction)
{
    int16 x0 = topLeftX;
    int16 y0 = topLeftY;
    int16 x1 = topLeftX + width;
    int16 y1 = topLeftY + height;

    // Clipping
    if (x0 < 0) x0 = 0;
    if (y0 < 0) y0 = 0;
    if (x1 > TFT_WIDTH)  x1 = TFT_WIDTH;
    if (y1 > TFT_HEIGHT) y1 = TFT_HEIGHT;

    if ((x0 >= x1) || (y0 >= y1)) return;

    setAddrWindow(x0, y0, x1 - 1, y1 - 1);

    for (int16 j = y0; j < y1; j++)
    {
        for (int16 i = x0; i < x1; i++)
        {
            uint16 color = callback(i - x0, j - y0);

            tftWriteData(color >> 8);
            tftWriteData(color & 0xff);
        }
    }

    tftWriteCommand(ILI9341_NOP); // just in case?

    setAddrWindow(0, 0, TFT_WIDTH - 1, TFT_HEIGHT - 1);
    tftWriteCommand(ILI9341_NOP);
}

void drawRectangle(uint16 x, uint16 y, uint16 width, uint16 height, uint16 color)
{
    setAddrWindow(x, y, x + width - 1, y + height - 1);

    for (int16 i = 0; i < height; i++)
    {
        for (int16 j = 0; j < width; j++)
        {
            tftWriteData(color >> 8);
            tftWriteData(color & 0xff);

        }
    }

    tftWriteCommand(ILI9341_NOP); // just in case?

    // Something weird happens when I call Mikroelektronika TFT_* functions if I don't do this.
    setAddrWindow(0, 0, TFT_WIDTH - 1, TFT_HEIGHT - 1);
    tftWriteCommand(ILI9341_NOP);
}


uint16 getScanline()
{
    uint16 scanline = 0;
    tftWriteCommand(0x45);
    CyDelayUs(8);
    tftReadData(); // dummy
    scanline = (uint16)tftReadData() << 8;
    scanline += (uint16)tftReadData();

    return scanline;
}

void waitForScanline(uint16 target)
{
    while (true)
    {
        uint16 scanline = getScanline();
        if (scanline == target)
        {
            break;
        }
    }
}
#endif
