#include <project.h>
#include <utils.h>
#include "time_psoc.h"

// TODO: Safety
// TODO: Check everything's working fine
//       Maybe add a lower priority interrupt routine

static bool8 inited = false;
static uint32 interruptPeriodInMs = 1;
static Timestamp timestamp = 0;

typedef struct
{
    void (*callback)();
    uint8 periodInMs;
    uint8 counter;
}
CallbackInfo;

CallbackInfo callInfos[2];
int callbackCount = 0;

// I don't like this TODO: ensure this isn't too costly
CY_ISR(timeInterrupt)
{
    timestamp++; // It's OK if it overflows

    for (int i = 0; i < callbackCount; i++)
    {
        CallbackInfo *callbackInfo = callInfos + i;
        if (callbackInfo->callback)
        {
            callbackInfo->counter++;
            if (callbackInfo->counter >= callbackInfo->periodInMs)
            {
                callbackInfo->counter = 0;
                callbackInfo->callback();
            }
        }
    }
}

void disableTimeInterrupts()
{
    if (inited) Timer_Int_Disable();
}

void enableTimeInterrupts()
{
    if (inited) Timer_Int_Enable();
}

static void initIfNotInited()
{
    if (!inited)
    {
        Timer_Int_StartEx(timeInterrupt); // TODO: bug
        CyGlobalIntEnable;
        inited = true;
    }
}

void setTimerCallback(void (*callback)(), uint8 periodInMs)
{
    if (callbackCount < arrayCount(callInfos))
    {
        CallbackInfo *callbackInfo = &callInfos[callbackCount];
        callbackInfo->periodInMs = periodInMs;
        callbackInfo->callback = callback;

        disableTimeInterrupts();
        callbackCount++;
        enableTimeInterrupts();
    }

    initIfNotInited();
}

void delayMs(uint32 ms)
{
    CyDelay(ms);
}

Timestamp getTimestamp()
{
    disableTimeInterrupts();
    Timestamp t = timestamp;
    enableTimeInterrupts();

    return t;
}

uint32 msSince(Timestamp t)
{
    Timestamp currentT = getTimestamp();
    uint32 ms = currentT - t; // I believe this handles wrapping correctly
    return ms;
}

float secondsSince(Timestamp t)
{
    return (float)msSince(t) / 1000.0f;
}
